import socket

from socket_project import config
from socket_project.utils.exceptions import ConnectionRequiredException


class BaseServer:
    def __init__(self, host: str, port: int):
        self._host = host
        self._port = port
        self._sock = None
    
    def get_host(self) -> str:
        return self._host
    
    def get_port(self) -> int:
        return self._port

    def _requires_sock(func):
        """ Decorator for validating that a certain method requires self._sock variable to not be None """
        def wrapper(self, *args, **kwargs):
            if self._sock is None:
                raise ConnectionRequiredException("Connection is required! Make sure you called the appropriate method before interaction.")
            func(self, *args, **kwargs)

        return wrapper

    def _socket_initializer(func):
        """ Decorator for those methods which initialize self._sock variable before execution """
        def wrapper(self, *args, **kwargs):
            if self._sock is None:
                self._sock = socket.socket(config.ADDRESS_FAMILY, config.SOCKET_TYPE)
            func(self, *args, **kwargs)

        return wrapper



import mimetypes

from socket_project import config


class FileHandler:
    def __init__(self, filepath: str):
        self._path = filepath
        with open(self._path, 'r') as f:
            self._contents = "".join(f.readlines())

    @property
    def contents(self) -> str:
        return self._contents

    @staticmethod
    def render_files(msg_utf8: list[str]) -> bytes:
        """ :param msg_utf8  msg.decode('utf-8').split('\r\n') """

        header = 'HTTP/1.1 200 OK\n'
        filename = msg_utf8[0].split(" ")[1].lstrip("/")
        filename = filename if filename != '' else 'index.html'

        try:
            with open(config.FRONTENDS_FOLDER + filename, 'rb') as f:
                response = f.read()
        except FileNotFoundError:
            with open(config.FRONTENDS_FOLDER + "404.html", 'rb') as f:
                response = f.read()

        mimetype = mimetypes.guess_type(filename)[0]
        mimetype = mimetype if mimetype is not None else 'text/html'

        header += 'Content-Type: ' + str(mimetype) + '\n\n'

        return header.encode() + response
